# coding: utf-8

from __future__ import absolute_import
from __future__ import unicode_literals

import os
import subprocess
from base64 import urlsafe_b64encode
from tempfile import NamedTemporaryFile as Temporary


class PKCS7Signer(object):

    """
    Класс реализующий PKCS7 подпись.
    
    :param certificate_file: Путь до сертификата.
    :type certificate_file: str or unicode
    
    :param private_key_file: Путь до приватного ключа.
    :type private_key_file: str or unicode
    """

    def __init__(self, certificate_file, private_key_file):
        self._check_openssl()
        self.certificate_file = certificate_file
        self.private_key_file = private_key_file

    def __call__(self, params):
        """Подписывает параметры и возвращает подпись.
        
        :param params: Параметры запроса.
        :return: Результат подписи параметров преобразованные в base64.
        :rtype: str
        """
        text_to_sign = self._get_text_to_sign(params)

        command = (
            'openssl smime -sign -md sha256 -in {source} -signer {cert} '
            '-inkey {key} -out {destination} -outform DER'
        )

        with Temporary('w') as source, Temporary('wb+') as destination:
            source.write(text_to_sign)
            source.seek(0)

            os.system(command.format(
                source=source.name,
                destination=destination.name,
                cert=self.certificate_file,
                key=self.private_key_file
            ))
            sign = destination.read()

        return urlsafe_b64encode(sign)

    @staticmethod
    def _get_text_to_sign(params):
        """Метод возвращает текст для подписывания."""
        items_to_sign = ('scope', 'timestamp', 'client_id', 'state')
        return ''.join(params.get(i, '') for i in items_to_sign)

    @staticmethod
    def _check_openssl():
        """Метод проверяет, установлен ли OpenSSL."""
        try:
            subprocess.check_output('openssl version', shell=True)
        except subprocess.CalledProcessError:
            raise OSError('OpenSSL is not installed')
